/* GNU FaXile - fax image viewer
 *
 * Copyright (C) 2000 Wolfgang Sourdeau
 *
 * Author: Wolfgang Sourdeau <wolfgang@ultim.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdio.h>

#include "tiffimages.h"
#include "viewer.h"
#include "cursors.h"

#include "pixmaps/left_arrow.xpm"
#include "pixmaps/right_arrow.xpm"
#include "pixmaps/zoom_in.xpm"
#include "pixmaps/zoom_out.xpm"
#include "pixmaps/reverse.xpm"
#include "pixmaps/printer.xpm"

float zoom_factor[] = {0.33, 0.50, 0.75, 1.0, 1.414, 2.0, 3.0};

extern void draw_page (ViewerData *viewer_data);
extern void print_cb (GtkWidget *widget, ViewerData *viewer_data);
extern GtkWidget *thumbs_fixed_new (FaxFile *fax_file);
extern void add_thumbs (GtkWidget *fixed, ViewerData *viewer_data);

static gint windows_count;
static guint app_width, app_height;

gint motion_notify_event_cb (GtkWidget *widget,
		       GdkEventMotion *event,
		       ViewerData *viewer_data)
{
	GtkAdjustment *adj_x, *adj_y;
	GdkModifierType mods;
	gint x, y, valx, valy, valx_up_lim, valy_up_lim;
	gint ret_code;

	if (viewer_data->motion_data->dragging) {
		adj_x = gtk_viewport_get_hadjustment (
			GTK_VIEWPORT
			(viewer_data->page_area->parent->parent));
		adj_y = gtk_viewport_get_vadjustment (
			GTK_VIEWPORT
			(viewer_data->page_area->parent->parent));

		if (event->is_hint) 
			gdk_window_get_pointer
				(widget->window, &x, &y, &mods);
		else {
			x = event->x;
			y = event->y;
		}

		valx = viewer_data->motion_data->orig_adj_x
			+ viewer_data->motion_data->orig_x - x;
		valy = viewer_data->motion_data->orig_adj_y
			+ viewer_data->motion_data->orig_y - y;

		valx_up_lim = adj_x->upper - 2 * adj_x->page_increment;
		valy_up_lim = adj_y->upper - 2 * adj_y->page_increment;

		if (valx < 0) {
			valx = 0;
			viewer_data->motion_data->orig_x =
				x - viewer_data->motion_data->orig_adj_x;
		} else if (valx > valx_up_lim) {
			valx = valx_up_lim;
			viewer_data->motion_data->orig_x =
				x + valx_up_lim
				- viewer_data->motion_data->orig_adj_x;
		}

		if (valy < 0) {
			valy = 0;
			viewer_data->motion_data->orig_y =
				y - viewer_data->motion_data->orig_adj_y;
		} else if (valy > valy_up_lim) {
			valy = valy_up_lim;
			viewer_data->motion_data->orig_y =
				y + valy_up_lim
				- viewer_data->motion_data->orig_adj_y;
		}

		gtk_adjustment_set_value
			(adj_x, valx); 
		gtk_adjustment_set_value
			(adj_y, valy);

		ret_code = TRUE;
	} else
		ret_code = FALSE;

	return ret_code;
}

guint button_release_event_cb (GtkWidget *widget,
			      GdkEventButton *event,
			      ViewerData *viewer_data)
{
	GdkCursor *cursor;
	guint ret_code;

	if (viewer_data->motion_data->dragging &&
	    event->button == 1) {
		viewer_data->motion_data->dragging = FALSE;
		cursor = cursor_get (widget->window,
				     CURSOR_HAND_OPEN);
		gdk_pointer_ungrab (event->time);
		gdk_cursor_destroy (cursor);

		ret_code = TRUE;
	} else
		ret_code = FALSE;

	return ret_code;
}

guint button_press_event_cb (GtkWidget *widget,
			    GdkEventButton *event,
			    ViewerData *viewer_data)
{
	GdkCursor *cursor;
	GtkAdjustment *adj_x, *adj_y;
	guint ret_code;
	GdkModifierType mods;

	if (!viewer_data->motion_data->dragging &&
	    event->button == 1) {
		adj_x = gtk_viewport_get_hadjustment (
			GTK_VIEWPORT
			(viewer_data->page_area->parent->parent));
		adj_y = gtk_viewport_get_vadjustment (
			GTK_VIEWPORT
			(viewer_data->page_area->parent->parent));

		viewer_data->motion_data->dragging = TRUE;
		gdk_window_get_pointer
			(viewer_data->viewer_window->window,
			&viewer_data->motion_data->orig_x,
			&viewer_data->motion_data->orig_y,
			&mods);

		viewer_data->motion_data->orig_adj_x = adj_x->value;
		viewer_data->motion_data->orig_adj_y = adj_y->value;

		cursor = cursor_get (widget->window,
				     CURSOR_HAND_CLOSED);
		gdk_pointer_grab (viewer_data->viewer_window->window,
				  FALSE,
				  (GDK_POINTER_MOTION_MASK
				   | GDK_POINTER_MOTION_HINT_MASK
				   | GDK_BUTTON_RELEASE_MASK),
				  NULL,
				  cursor,
				  event->time);
		gdk_cursor_destroy (cursor);

		ret_code = TRUE;
	} else
		ret_code = FALSE;

	return ret_code;
}

void nextpage_cb (GtkWidget *widget, ViewerData *viewer_data)
{
	viewer_data->current_page = viewer_data->current_page->next;
	draw_page (viewer_data);
}

void prevpage_cb (GtkWidget *widget, ViewerData *viewer_data)
{
	viewer_data->current_page = viewer_data->current_page->prev;
	draw_page (viewer_data);
}	

void zoomout_cb (GtkWidget *widget, ViewerData *viewer_data)
{
	viewer_data->zoom_index--;
	draw_page (viewer_data);
}

void zoomin_cb (GtkWidget *widget, ViewerData *viewer_data)
{
	viewer_data->zoom_index++;
	draw_page (viewer_data);
}

void reverse_cb (GtkWidget *widget, ViewerData *viewer_data)
{
	viewer_data->reverse_page = !(viewer_data->reverse_page);
	draw_page (viewer_data);
}

void closeviewer_cb (GtkWidget *widget, ViewerData *viewer_data)
{
	ti_close_fax_file (viewer_data->fax_file);
	gdk_pixmap_unref (viewer_data->page_pixmap);
	g_free (viewer_data->th_buttons);
	g_free (viewer_data->motion_data);

	windows_count--;

	if (windows_count == 0)
		gtk_main_quit();
}

GtkWidget *button_bar_new (ViewerData *viewer_data)
{
	GtkWidget *new_bbar;
	GdkPixmap *pix_next, *pix_prev,
		  *pix_zin, *pix_zout,
		  *pix_reverse, *pix_print;
	GtkWidget *icon_next, *icon_prev,
		  *icon_zin, *icon_zout,
		  *icon_reverse, *icon_print;
	GtkWidget *but_reverse, *but_print;
	GdkBitmap *mask;

	new_bbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL,
        			   GTK_TOOLBAR_ICONS);

	pix_next =
		gdk_pixmap_create_from_xpm_d
		(viewer_data->viewer_window->window,
		 &mask,
		 &(viewer_data->viewer_window->style->bg[GTK_STATE_NORMAL]), 
		 (gchar **) right_arrow_xpm);

	icon_next = gtk_pixmap_new (pix_next, mask);
	gtk_widget_show (icon_next);
	viewer_data->bb_buttons[NEXT_PAGE] = gtk_button_new ();
	gtk_container_add
		(GTK_CONTAINER(viewer_data->bb_buttons[NEXT_PAGE]),
		 icon_next);
	gtk_signal_connect
		(GTK_OBJECT(viewer_data->bb_buttons[NEXT_PAGE]),
		 "clicked", GTK_SIGNAL_FUNC(nextpage_cb), viewer_data);
	
	pix_prev = gdk_pixmap_create_from_xpm_d
		(viewer_data->viewer_window->window,
		 &mask,
		 &(viewer_data->viewer_window->style->bg[GTK_STATE_NORMAL]),
		 (gchar **)left_arrow_xpm); 
	icon_prev = gtk_pixmap_new (pix_prev, mask);
	gtk_widget_show (icon_prev);
	viewer_data->bb_buttons[PREV_PAGE] = gtk_button_new();
	gtk_container_add
		(GTK_CONTAINER(viewer_data->bb_buttons[PREV_PAGE]),
		 icon_prev);
	gtk_signal_connect
		(GTK_OBJECT(viewer_data->bb_buttons[PREV_PAGE]),
		 "clicked", GTK_SIGNAL_FUNC(prevpage_cb), viewer_data);

	pix_zin = gdk_pixmap_create_from_xpm_d
		(viewer_data->viewer_window->window,
		 &mask,
		 &(viewer_data->viewer_window->style->bg[GTK_STATE_NORMAL]),
		 (gchar **)zoom_in_xpm);
	icon_zin = gtk_pixmap_new (pix_zin, mask);
	gtk_widget_show (icon_zin);
	viewer_data->bb_buttons[ZOOM_IN] = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER
			   (viewer_data->bb_buttons[ZOOM_IN]),
			   icon_zin);
	gtk_signal_connect
		(GTK_OBJECT (viewer_data->bb_buttons[ZOOM_IN]),
		 "clicked", GTK_SIGNAL_FUNC (zoomin_cb), viewer_data);

	pix_zout =
		gdk_pixmap_create_from_xpm_d
		(viewer_data->viewer_window->window,
		 &mask,
		 &(viewer_data->viewer_window->style->bg[GTK_STATE_NORMAL]),
		 (gchar **)zoom_out_xpm);
	icon_zout = gtk_pixmap_new(pix_zout, mask);
	gtk_widget_show (icon_zout);
	viewer_data->bb_buttons[ZOOM_OUT] = gtk_button_new();
	gtk_container_add (GTK_CONTAINER
			   (viewer_data->bb_buttons[ZOOM_OUT]),
			   icon_zout);
	gtk_signal_connect (GTK_OBJECT
			    (viewer_data->bb_buttons[ZOOM_OUT]),
			    "clicked",
			    GTK_SIGNAL_FUNC(zoomout_cb),
			    viewer_data);

	pix_reverse = gdk_pixmap_create_from_xpm_d
		(viewer_data->viewer_window->window,
		 &mask,
		 &(viewer_data->viewer_window->style->bg[GTK_STATE_NORMAL]),
		 (gchar **) reverse_xpm);
	icon_reverse = gtk_pixmap_new (pix_reverse, mask);
	gtk_widget_show (icon_reverse);
	but_reverse = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER (but_reverse), icon_reverse);
	gtk_signal_connect (GTK_OBJECT (but_reverse), "clicked",
			    GTK_SIGNAL_FUNC (reverse_cb),
			    viewer_data);
	gtk_widget_show (but_reverse);

	pix_print = gdk_pixmap_create_from_xpm_d
		(viewer_data->viewer_window->window, 
		 &mask,
		 &(viewer_data->viewer_window->style->bg[GTK_STATE_NORMAL]),
		 (gchar **) printer_xpm);
	icon_print = gtk_pixmap_new (pix_print, mask);
	gtk_widget_show (icon_print);
	but_print = gtk_button_new ();
	gtk_container_add (GTK_CONTAINER(but_print), icon_print);
	gtk_signal_connect (GTK_OBJECT (but_print), "clicked",
			    GTK_SIGNAL_FUNC (print_cb),
			    viewer_data);
	gtk_widget_show (but_print);

	gtk_toolbar_append_widget(GTK_TOOLBAR (new_bbar),
				viewer_data->bb_buttons[PREV_PAGE],
				_("Go to previous page"),
				NULL);
	gtk_toolbar_append_widget(GTK_TOOLBAR (new_bbar),
				viewer_data->bb_buttons[NEXT_PAGE],
				_("Go to next page"),
				NULL);
	gtk_toolbar_append_space (GTK_TOOLBAR (new_bbar));
	gtk_toolbar_append_widget (GTK_TOOLBAR (new_bbar),
				   viewer_data->bb_buttons[ZOOM_IN],
				   _("Zoom in"),
				   NULL);
	gtk_toolbar_append_widget (GTK_TOOLBAR (new_bbar),
				   viewer_data->bb_buttons[ZOOM_OUT],
				   _("Zoom out"),
				   NULL);
	gtk_toolbar_append_space (GTK_TOOLBAR (new_bbar));
	gtk_toolbar_append_widget (GTK_TOOLBAR (new_bbar),
				   but_reverse,
				   _("Reverse image"),
				   NULL);
	gtk_toolbar_append_widget( GTK_TOOLBAR (new_bbar),
				   but_print,
				   _("Print fax"),
				   NULL);

	gtk_toolbar_set_space_style (GTK_TOOLBAR (new_bbar),
				     GTK_TOOLBAR_SPACE_LINE);
	gtk_toolbar_set_space_size (GTK_TOOLBAR (new_bbar), 10);

        gtk_button_set_relief (GTK_BUTTON
			       (viewer_data->bb_buttons[PREV_PAGE]),
			       GTK_RELIEF_NONE);
        gtk_button_set_relief (GTK_BUTTON
			       (viewer_data->bb_buttons[NEXT_PAGE]), 
			       GTK_RELIEF_NONE);
        gtk_button_set_relief (GTK_BUTTON
			       (viewer_data->bb_buttons[ZOOM_IN]),
			       GTK_RELIEF_NONE);
        gtk_button_set_relief (GTK_BUTTON
			       (viewer_data->bb_buttons[ZOOM_OUT]),
			       GTK_RELIEF_NONE);
	gtk_button_set_relief (GTK_BUTTON (but_reverse),
			       GTK_RELIEF_NONE);
	gtk_button_set_relief (GTK_BUTTON (but_print),
			       GTK_RELIEF_NONE);

	gtk_widget_show (viewer_data->bb_buttons[PREV_PAGE]);
	gtk_widget_show (viewer_data->bb_buttons[NEXT_PAGE]);
	gtk_widget_show (viewer_data->bb_buttons[ZOOM_IN]);
	gtk_widget_show (viewer_data->bb_buttons[ZOOM_OUT]);

	return new_bbar; 
}

void viewer_window_new(gchar *title, ViewerData *viewer_data)
{
	GtkWidget *vbox, *view_hbox, *button_bar;
	GtkWidget *page_table, *page_sc_win;
	GtkWidget *thumbs_fixed, *thumbs_sc_win;
	GdkCursor *cursor;

	viewer_data->viewer_window =
		gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (viewer_data->viewer_window),
			      title);
	gtk_signal_connect (GTK_OBJECT (viewer_data->viewer_window),
			    "destroy",
			    GTK_SIGNAL_FUNC (closeviewer_cb),
			    viewer_data);
	gtk_window_set_default_size (GTK_WINDOW
				     (viewer_data->viewer_window),
				     app_width, app_height);
	gtk_container_set_border_width (GTK_CONTAINER
					(viewer_data->viewer_window),
					5);
	gtk_widget_show (viewer_data->viewer_window);

	vbox = gtk_vbox_new (FALSE, 3);	
	gtk_container_add (GTK_CONTAINER (viewer_data->viewer_window),
			   vbox);

	button_bar = button_bar_new (viewer_data);
	gtk_box_pack_start (GTK_BOX (vbox), button_bar,
			    FALSE, FALSE, 1);
	gtk_widget_show (button_bar);

	view_hbox = gtk_hbox_new (FALSE, 2);
	gtk_box_pack_start (GTK_BOX (vbox), view_hbox, TRUE, TRUE, 5);

	gtk_widget_show (vbox);

	thumbs_sc_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (thumbs_sc_win),
					GTK_POLICY_NEVER,
					GTK_POLICY_ALWAYS);
	gtk_widget_show (thumbs_sc_win);

	thumbs_fixed = thumbs_fixed_new (viewer_data->fax_file);
	gtk_widget_show (thumbs_fixed);

	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW
					       (thumbs_sc_win),
					       thumbs_fixed);
	gtk_box_pack_start (GTK_BOX (view_hbox), thumbs_sc_win,
			    FALSE, FALSE, 0);

	page_sc_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (page_sc_win),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start (GTK_BOX(view_hbox),
			    page_sc_win,
			    TRUE,
			    TRUE, 2);
	gtk_widget_show (view_hbox);

	page_table = gtk_table_new (1, 1, FALSE);
	gtk_scrolled_window_add_with_viewport
		(GTK_SCROLLED_WINDOW (page_sc_win),
		 page_table);

	gtk_widget_show (page_table);
	gtk_widget_show (page_sc_win);

	viewer_data->page_area = gtk_drawing_area_new ();
	gtk_table_attach (GTK_TABLE (page_table),
			  viewer_data->page_area,
			  0, 1, 0, 1,
			  GTK_EXPAND | GTK_SHRINK,
			  GTK_EXPAND | GTK_SHRINK,
			  1, 1);

	gtk_widget_show (viewer_data->page_area);

	while (gtk_events_pending ())
		gtk_main_iteration ();

	add_thumbs (thumbs_fixed, viewer_data);

	while (gtk_events_pending ())
		gtk_main_iteration ();

	cursor = cursor_get (viewer_data->page_area->window, CURSOR_HAND_OPEN);
	gdk_window_set_cursor (viewer_data->page_area->window, cursor);
	gdk_cursor_destroy (cursor);

	viewer_data->current_page = viewer_data->fax_file->first;
	draw_page(viewer_data);

	gdk_window_set_events
		(viewer_data->viewer_window->window,
		 gdk_window_get_events
		 (viewer_data->viewer_window->window)
		 | GDK_EXPOSURE_MASK
		 | GDK_BUTTON_RELEASE_MASK);

	gdk_window_set_events
		(viewer_data->page_area->window,
		gdk_window_get_events (viewer_data->page_area->window)
		| GDK_BUTTON_PRESS_MASK);

	gtk_signal_connect (GTK_OBJECT (viewer_data->page_area),
			    "button_press_event",
			    GTK_SIGNAL_FUNC (button_press_event_cb),
			    viewer_data);
	gtk_signal_connect (GTK_OBJECT (viewer_data->viewer_window),
			    "button_release_event",
			    GTK_SIGNAL_FUNC (button_release_event_cb),
			    viewer_data);
	gtk_signal_connect (GTK_OBJECT (viewer_data->viewer_window),
			    "motion_notify_event",
			    GTK_SIGNAL_FUNC (motion_notify_event_cb),
			    viewer_data);

	windows_count++;
}

gint fax_viewer_new (gchar *file_name)
{
	ViewerData *new_viewer;
	FaxFile *fax_file;
	gchar *title;
	gint ret_code;

	fax_file = ti_open_fax_file (file_name);
	if (fax_file != NULL) {
		new_viewer = g_malloc (sizeof(ViewerData));
		new_viewer->file_name = file_name;
		new_viewer->fax_file = fax_file;
		new_viewer->current_page = ti_seek_fax_page (fax_file, 0);
		new_viewer->zoom_index = 3;
		new_viewer->reverse_page = FALSE;
		new_viewer->current_page = NULL;
		new_viewer->page_area = NULL;
		new_viewer->page_pixmap = NULL;
		new_viewer->print_dialog = NULL;
		new_viewer->motion_data = g_malloc (sizeof
						    (MotionData));
		new_viewer->motion_data->dragging = FALSE;

		title = g_strdup_printf ("%s %s",
				_("GNU FaXile - Viewer"),
				VERSION);

		viewer_window_new(title, new_viewer);
		g_free (title);

		ret_code = 1;
	} else
		ret_code = 0;

	return ret_code;
}

#ifdef __WIN32__
#warning Strange hacks are happening...
void dummy_output_handler (const gchar* log_domain,
			   GLogLevelFlags log_level,
			   const gchar *message,
			   gpointer user_data)
{
}
#endif

gchar *list_to_text (GSList *string_list)
{
	gchar *ret_str, *cur_pos;
	GSList *cur_str;
	guint count, str_c, length;

	length = 0;
	str_c = 0;

	cur_str = string_list;

	while (cur_str) {
		length += strlen ((char*) cur_str->data) + 1;
		str_c++;
		cur_str = cur_str->next;
	}

	if (str_c) {
		ret_str = g_malloc (length);
		cur_pos = ret_str;
		cur_str = string_list;

		for (count = 1; count < str_c; count++) {
			strcpy (cur_pos, (char*) cur_str->data);
			cur_pos += strlen ((char*) cur_str->data);
			sprintf (cur_pos, "\n");
			cur_pos ++;
			cur_str = cur_str->next;
		}

		sprintf (cur_pos, "%s", (char*) cur_str->data);
	} else
		ret_str = NULL;

	return ret_str;
}

gint fail_win_destroy_cb (GtkWidget* window, gpointer null)
{
	windows_count--;

	if (!windows_count)
		gtk_main_quit ();

	return FALSE;
}

void display_failures (GSList *fail_list)
{
	gchar *list_text;
	GtkWidget *fail_window, *vbox, *list_lbl, *msg_lbl,
		*separator, *ok_but; 

	list_text = list_to_text (fail_list);
  	fail_window = gtk_window_new (GTK_WINDOW_DIALOG);
  	gtk_window_set_modal (GTK_WINDOW (fail_window), TRUE);
	gtk_window_set_title (GTK_WINDOW (fail_window),
			      _("Something strange happened..."));
	gtk_window_set_policy (GTK_WINDOW (fail_window),
			       FALSE, FALSE, TRUE);

	gtk_container_set_border_width (GTK_CONTAINER (fail_window),
					10);

 	vbox = gtk_vbox_new (FALSE, 1);
	gtk_container_add (GTK_CONTAINER (fail_window), vbox);
	gtk_widget_show (vbox);

	msg_lbl = gtk_label_new (_("The following file(s) could not"
				   " be opened.\n"
				   "The reason for this might be"
				   " that :\n"
				   " * it/they were not fax tiff"
				   " files or\n"
				   " * that you don't have the right"
				   " to read them or\n"
				   " * simply that it/they don't"
				   " exist."));
	gtk_label_set_justify (GTK_LABEL (msg_lbl),
			       GTK_JUSTIFY_LEFT);
	gtk_box_pack_start (GTK_BOX (vbox), msg_lbl,
			    FALSE, FALSE, 0);
	gtk_widget_show (msg_lbl);

	separator = gtk_hseparator_new ();
	gtk_box_pack_start (GTK_BOX (vbox), separator,
			    FALSE, FALSE, 5);
	gtk_widget_show (separator);

	list_lbl = gtk_label_new (list_text);
	gtk_label_set_justify (GTK_LABEL (list_lbl),
			       GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap (GTK_LABEL (list_lbl), TRUE);
	gtk_box_pack_start (GTK_BOX (vbox), list_lbl,
			    TRUE, FALSE, 0);
	gtk_widget_show (list_lbl);

	ok_but = gtk_button_new_with_label (_("I am sorry"));

	gtk_box_pack_start (GTK_BOX (vbox), ok_but, FALSE, FALSE, 2);
	gtk_signal_connect_object (GTK_OBJECT (ok_but), "clicked",
				   GTK_SIGNAL_FUNC
				   (gtk_widget_destroy),
				   GTK_OBJECT (fail_window));
	gtk_widget_show (ok_but);

	windows_count++;
	gtk_signal_connect (GTK_OBJECT (fail_window), "destroy",
			    GTK_SIGNAL_FUNC (fail_win_destroy_cb),
			    NULL);

	gtk_widget_show (fail_window);
}

gint file_dlg_ok_cb (GtkWidget *ok_button, GtkWidget *file_dlg)
{
	gchar *file_name;
	guint failures;
	GSList *fail_list;

	file_name = g_strdup (gtk_file_selection_get_filename
			(GTK_FILE_SELECTION (file_dlg)));
	fail_list = NULL;

	if (file_name) {
		windows_count--;
		gtk_widget_destroy (file_dlg);
		if (!fax_viewer_new (file_name)) {
			fail_list = g_slist_append (fail_list, file_name);
			display_failures (fail_list);
			g_free (file_name);
			g_slist_free (fail_list);
		}
	}

	return FALSE;
}

void file_dialog ()
{
	GtkWidget *open_dialog;

	open_dialog =
		gtk_file_selection_new (_("Please choose a FAX "
					  "G3 file to open..."));
	gtk_signal_connect (
		GTK_OBJECT (GTK_FILE_SELECTION
			    (open_dialog)->ok_button),
		"clicked",
		GTK_SIGNAL_FUNC (file_dlg_ok_cb),
		open_dialog);
	gtk_signal_connect (
		GTK_OBJECT (GTK_FILE_SELECTION
			    (open_dialog)->cancel_button),
		"clicked",
		GTK_SIGNAL_FUNC (gtk_main_quit),
		NULL);

	windows_count++;

	gtk_widget_show (open_dialog);
}

#if defined (ENABLE_NLS) && defined (__WIN32__)
gchar *win32_locale_dir ()
{
	gchar *locale_dir;
	gchar file_name[MAX_PATH];
	gchar *sep;

	GetModuleFileName (NULL, file_name, sizeof (file_name));
	sep = strrchr (file_name, '\\');
	*sep = '\0';

	locale_dir = g_strdup_printf ("%s\\locale",
				      file_name);

	return locale_dir;
}
#endif

int main(int argc, char *argv[])
{
	int arg_count, scr_width, scr_height;
	guint failures;
	GSList *fail_list;

	windows_count = 0;
	arg_count = 1;

#ifdef ENABLE_NLS
#ifndef __WIN32__
	bindtextdomain (PACKAGE, LOCALEDIR);
#else
	bindtextdomain (PACKAGE, win32_locale_dir ());
#endif
	textdomain (PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	TIFFSetErrorHandler (NULL);

#if defined(__WIN32__) && !defined(DEBUG)
	g_log_set_handler ("Gtk",
			   G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING,
			   (GLogFunc) dummy_output_handler,
			   NULL);
#endif

	fail_list = NULL;

	/* Not a perfect mechanism, but acceptable for now. */
	scr_width = gdk_screen_width ();
	scr_height = gdk_screen_height ();
	if (scr_width > 800 && scr_height > 600) {
		app_width = 800;
		app_height = 600;
	} else {
		app_width = scr_width - 10;
		app_height = scr_height - 100;
	}
	
	if (argc == 1)
		file_dialog ();
	else
		while (argv[arg_count] != NULL) {
			if (!fax_viewer_new (argv[arg_count]))
				fail_list = g_slist_append 
					(fail_list, argv[arg_count]);
			arg_count++;
		}

	failures = g_slist_length (fail_list);

	if (failures)
		display_failures (fail_list);

	if (windows_count > 0) {
		g_slist_free (fail_list);
		gtk_main();
	}

	return 0;
}
